# -*- coding: utf-8 -*-
"""Module holds functions that read context from various formats of files"""

from fcatng.readwrite.tab_separated import *
from fcatng.readwrite.comma_separated import *
from fcatng.readwrite.cxt import *
from fcatng.readwrite.dot import *
from fcatng.readwrite.xml_ import *
from fcatng.readwrite.fimi import *
